#ifndef LATTICEDESCRIPTORSQR_HH_
#define LATTICEDESCRIPTORSQR_HH_


#include "LatticeDescriptor.hh"

namespace biu
{

		/**
		 * Lattice property handler for the integer square lattice.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	
	class LatticeDescriptorSQR : public LatticeDescriptor
	{
	protected:
		virtual unsigned int getNeighborDataSize() const;
		virtual const NeighborData *getNeighborData() const;
		virtual unsigned int getAutomorphismDataSize() const;
		virtual const AutomorphismData *getAutomorphismData() const;
	public:
		LatticeDescriptorSQR();
		virtual ~LatticeDescriptorSQR();
			//! Returns whether or not two points in the lattice are neighbored
			//! using a specialised implementation for the square lattice
			//! @param first the first point
			//! @param second the second point
			//! @return true if (second-first) is element of the neighborhood,
			//!         false otherwise
		virtual bool areNeighbored( const IntPoint &first, 
									const IntPoint &second ) const;
	};

} // namespace biu

#endif /*LATTICEDESCRIPTORSQR_HH_*/
