#ifndef BIU_LATTICEDESCRIPTORCKW_HH_
#define BIU_LATTICEDESCRIPTORCKW_HH_

#include "biu/LatticeDescriptor.hh"

namespace biu
{

	/*!
	 * Lattice descriptor of the chess knight walk lattice also known as the
	 * 210 lattice.
	 * 
	 * The neighborhood vertices are all permutations of (2,1,0) and its 
	 * negations. Thus 24 neighboring vectors are possible.
	 * 
	 * @author Martin Mann http://www.bioinf.uni-freiburg.de/~mmann/
	 * 
	 */
	class LatticeDescriptorCKW : public LatticeDescriptor
	{
	protected:

			// abstract function implementation
		virtual unsigned int getNeighborDataSize() const;
		virtual const NeighborData *getNeighborData() const;
		virtual unsigned int getAutomorphismDataSize() const;
		virtual const AutomorphismData *getAutomorphismData() const;
	public:
	
		 /*! Construction
		  */
		LatticeDescriptorCKW();
		
		 /*! Copy construction.
		  * @param toCopy the CKW lattice descriptor to copy
		  */
		LatticeDescriptorCKW( const LatticeDescriptorCKW& toCopy);
		
		 /*! Destruction
		  */
		virtual ~LatticeDescriptorCKW();
		
			//! Returns whether or not two points in the lattice are neighbored
			//! using a specialised implementation for the chess knight walk
			//! lattice
			//! @param first the first point
			//! @param second the second point
			//! @return true if (second-first) is element of the neighborhood,
			//!         false otherwise
		virtual bool areNeighbored( const IntPoint &first, 
									const IntPoint &second ) const;
	};

}

#endif /*LATTICEDESCRIPTORCKW_HH_*/
